#
# FtpCube
# Copyright (C) 2001 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import gtk, GTK, GtkExtra
import os, pickle
import main, app, connectwin

class BookmarkWin:

	headings = [ 'Description', 'Host', 'Username', 'Directory' ]

	def __init__ (self, config_dir):

		self.window = gtk.GtkWindow ()
		self.window.set_title ("ftpcube - Bookmarks")
		self.window.set_usize (750, 500)
		self.window.show ()

		packer = gtk.GtkPacker ()
		packer.show ()
		self.window.add (packer)

		self.menu_init (packer)
		self.toolbar_init (packer)

		hbox = gtk.GtkHBox ()
		hbox.show ()
		packer.add (hbox, options=GTK.EXPAND+GTK.FILL_X+GTK.FILL_Y)
		pane = gtk.GtkHPaned ()
		pane.show ()
		hbox.add (pane)

		frame = gtk.GtkFrame ()
		frame.show ()
		pane.pack1 (frame)
		self.tree = gtk.GtkCTree ()
		self.tree.connect ("select_row", self.tree_select)
		self.tree.connect ("button_press_event", self.tree_popup_menu)
		self.tree.show ()
		frame.add (self.tree)

		frame = gtk.GtkFrame ()
		frame.show ()
		pane.pack2 (frame)
		scrolled = gtk.GtkScrolledWindow ()
		scrolled.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scrolled.show ()
		frame.add (scrolled)

		self.list = gtk.GtkCList (4, self.headings)
		self.list.connect ("select_row", self.select_items)
		self.list.connect ("button_press_event", self.list_popup_menu)
		# Set up list column defaults
		for i in range(len (self.headings)):
			self.list.set_column_width (i, 100)
			self.list.set_column_justification (i, gtk.JUSTIFY_LEFT)
		self.list.set_column_width (0, 200)
		self.list.show ()
		scrolled.add_with_viewport (self.list)

		# Compute the position of the pane divider
		pane.set_position (250)

		# Check if the bookmark directory in the config directory exists
		self.bookdir = config_dir
		path = os.path.join (self.bookdir, 'bookmarks')
		if not os.path.exists (path):
			try:
				os.mkdir (path)
				os.chmod (path, 0700)
			except OSError, strerror:
				GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ]))

		self.load_pixmaps ()
		# Now load the initial bookmarks directory into the tree
		node = self.tree.insert_node (None, None, [ "bookmarks" ],
						pixmap_closed = self.folder_pic,
						mask_closed = self.folder_mask,
						pixmap_opened = self.folder_pic,
						mask_opened = self.folder_mask,
						is_leaf = gtk.FALSE)
		self.load_bookmark_tree (os.path.join (self.bookdir, "bookmarks"), parent=node)
		self.tree.expand (node)
		self.tree.select (node)
		self.display_bookmarks ()

	def menu_init (self, container):

		ag = gtk.GtkAccelGroup ()
		item = gtk.GtkItemFactory (gtk.GtkMenuBar, "<main>", ag)
		self.window.add_accel_group (ag)

		site = self.menu_process_site
		edit = self.menu_process_edit

		item.create_items ([
			('/_Site',                 None,  None,        0,   '<Branch>'),
			('/Site/_Connect',         None,  site,        1,   ''),
			('/_Edit',                 None,  None,        0,   '<Branch>'),
			('/_Edit/Add _Folder',     None,  edit,        2,   ''),
			('/_Edit/Rename Folder',   None,  edit,        3,   ''),
			('/_Edit/Delete Folder',   None,  edit,        4,   ''),
			('/_Edit/<Separator>',     None,  None,        0,   '<Separator>'),
			('/_Edit/Add _Bookmark',   None,  edit,        5,   ''),
			('/_Edit/_Edit Bookmark',  None,  edit,        6,   ''),
			('/_Edit/Rename Bookmark', None,  edit,        7,   ''),
			('/_Edit/Delete Bookmark', None,  edit,        8,   ''),
		])
		mainmenu = item.get_widget ('<main>')

		# Now add the menu bar into the canvas
		menubox = gtk.GtkVBox ()
		menubox.pack_start (mainmenu, expand=gtk.FALSE)
		mainmenu.show ()
		menubox.show ()
		container.add (menubox, side=gtk.SIDE_TOP, anchor=gtk.ANCHOR_NW, options=GTK.FILL_X)

	def menu_process_site (self, action, widget):

		if action == 1:
			self.connect ()

	def menu_process_edit (self, action, widget):

		if action == 2:
			self.add_folder ()
		elif action == 3:
			self.rename_folder ()
		elif action == 4:
			self.remove_folder ()
		elif action == 5:
			self.add_bookmark ()
		elif action == 6:
			self.edit_bookmark ()
		elif action == 7:
			self.rename_bookmark ()
		elif action == 8:
			self.remove_bookmark ()

	def toolbar_init (self, container):

		toolbox = gtk.GtkHandleBox ()
		toolbar = gtk.GtkToolbar (gtk.ORIENTATION_HORIZONTAL, gtk.TOOLBAR_ICONS)
		toolbar.set_space_size (10)
		toolbar.set_space_style (GTK.TOOLBAR_SPACE_LINE)

		connect_pic, connect_mask = gtk.create_pixmap_from_xpm (self.window, None, os.path.join (main.ICONS_PREFIX, "quick_connect.xpm"))
		folder_pic, folder_mask = gtk.create_pixmap_from_xpm (self.window, None, os.path.join (main.ICONS_PREFIX, "add_folder.xpm"))
		add_pic, add_mask = gtk.create_pixmap_from_xpm (self.window, None, os.path.join (main.ICONS_PREFIX, "add_bookmark.xpm"))
		edit_pic, edit_mask = gtk.create_pixmap_from_xpm (self.window, None, os.path.join (main.ICONS_PREFIX, "edit_bookmark.xpm"))

		def connect_button (button, toolbar=toolbar, self=self):
			self.connect ()

		def folder_button (button, toolbar=toolbar, self=self):
			self.add_folder ()

		def add_button (button, toolbar=toolbar, self=self):
			self.add_bookmark ()

		def edit_button (button, toolbar=toolbar, self=self):
			self.edit_bookmark ()

		toolbar.append_item ("Connect", "Connect",
			"Connect",
			gtk.GtkPixmap (connect_pic, connect_mask),
			connect_button)
		toolbar.append_space ()
		toolbar.append_item ("Add Folder", "Add Folder",
			"Add Folder",
			gtk.GtkPixmap (folder_pic, folder_mask),
			folder_button)
		toolbar.append_item ("Add Bookmark", "Add Bookmark",
			"Add Bookmark",
			gtk.GtkPixmap (add_pic, add_mask),
			add_button)
		toolbar.append_item ("Edit Bookmark", "Edit Bookmark",
			"Edit Bookmark",
			gtk.GtkPixmap (edit_pic, edit_mask),
			edit_button)

		toolbar.show ()
		toolbox.add (toolbar)
		toolbox.show ()
		container.add (toolbox, side=gtk.SIDE_TOP, anchor=gtk.ANCHOR_NW, options=GTK.FILL_X)

	def destroy (self):

		self.window.hide ()
		self.window.destroy ()

	def tree_popup_menu (self, tree, event):

		if event.button == 3 and not tree.selection == []:
			opts = [
				('Rename Folder', None, lambda b, self=self: self.rename_folder ()),
				('Delete Folder', None, lambda b, self=self: self.remove_folder ()),
			]

			menu = GtkExtra.MenuFactory (GtkExtra.MENU_FACTORY_MENU)
			menu.add_entries (opts)
			menu.popup (None, None, None, event.button, event.time)
			menu.show ()

	def list_popup_menu (self, list, event):

		if event.button == 3 and not self.list.selection == [ ]:
			opts = [
				('/Edit Bookmark', None, lambda b, self=self: self.edit_bookmark ()),
				('/Rename Bookmark', None, lambda b, self=self: self.rename_bookmark ()),
				('/Delete Bookmark', None, lambda b, self=self: self.remove_bookmark ()),
			]

			menu = GtkExtra.MenuFactory (GtkExtra.MENU_FACTORY_MENU)
			menu.add_entries (opts)
			menu.popup (None, None, None, event.button, event.time)
			menu.show ()

	def connect (self):

		path = self.get_path_to_current_node ()
		name = self.list.get_text (self.list.selection.pop (0), 0)
		bookfile = os.path.join (path, name)
		options = self.load_bookmark (bookfile)

		# Check if we're breaking a connection and display the disconnect box
		# if we are
		if main.app.main_thread is None:
			connectwin.ConnectWin (main.app.initiate_connection, options)
		else:

			def callback (callback=main.app.initiate_connection, opts=options):

				main.app.main_thread.cancel_thread ()
				connectwin.ConnectWin (callback, opts)

			app.DisconnectBox (callback)

		self.destroy ()

	def select_items (self, list, r, c, event):

		if hasattr (event, 'type'):
			# Check for double clicks
			if event.type == 5:

				self.list.selection.append (r)
				self.connect ()

	def load_pixmaps (self):

		self.folder_pic, self.folder_mask = gtk.create_pixmap_from_xpm (self.list, None, os.path.join (main.ICONS_PREFIX, "folder.xpm"))

	def tree_select (self, tree, row, col, event):

		if hasattr(event, 'type'):
			if event.type == 5:
				node = self.tree.node_nth (row)
				self.tree.expand (node)

		self.display_bookmarks ()

	def add_folder (self):

		if not self.tree.selection:
			# Just use the top level
			node = None
			path = self.bookdir
			folder = GtkExtra.input_box (title="FtpCube - Add Bookmark Folder", message="Folder Name:")
		else:
			node = self.tree.selection.pop (0)
			path = self.get_path_to_node (node)
			folder = GtkExtra.input_box (title="FtpCube - Add Bookmark Folder", message="Folder Name:")

		if folder is not None:
			os.mkdir (os.path.join (path, folder), 0700)
			node = self.tree.insert_node (node, None, [ folder ],
					pixmap_closed = self.folder_pic,
					mask_closed = self.folder_mask,
					pixmap_opened = self.folder_pic,
					mask_opened = self.folder_mask,
					is_leaf = gtk.FALSE)

	def rename_folder (self):

		for node in self.tree.selection:

			# Skip if this is the root node
			if node == self.tree.base_nodes ().pop (0):
				continue

			path = self.get_path_to_node (node)
			new_folder = GtkExtra.input_box (title="FtpCube - Rename Folder", message="Rename to:")
			if new_folder is not None:
				new_path = os.path.join (os.path.dirname (path), new_folder)
				try:
					os.rename (path, new_path)
				except OSError, (errno, strerror):
					GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ]))
				parent = node.parent
				self.tree.remove_node (node)
				node = self.tree.insert_node (parent, None, [ new_folder ],
					pixmap_closed = self.folder_pic,
					mask_closed = self.folder_mask,
					pixmap_opened = self.folder_pic,
					mask_opened = self.folder_mask,
					is_leaf = gtk.FALSE)
				self.load_bookmark_tree (new_path, parent=node)

	def remove_folder (self):

		for node in self.tree.selection:

			# Skip if this is the root node
			if node == self.tree.base_nodes ().pop (0):
				continue

			path = self.get_path_to_node (node)
			try:
				os.rmdir (path)
			except OSError, (errno, strerror):
				GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ]))
				return
			self.tree.remove_node (node)

	def get_path_to_current_node (self):

		# Get the current folder
		if not self.tree.selection:
			return self.bookdir
		else:
			node = self.tree.selection.pop (0)
			return self.get_path_to_node (node)

	def get_path_to_node (self, node):

		if node.parent is None:
			return os.path.join (self.bookdir, (self.tree.node_get_pixtext (node, 0))[0])
		else:
			# Figure out current path
			path = (self.tree.node_get_pixtext (node, 0))[0]
			while node.parent is not None:
				node = node.parent
				path = os.path.join ((self.tree.node_get_pixtext (node, 0))[0], path)
			return os.path.join (self.bookdir, path)

	def add_bookmark (self):

		config_opts = main.app.load_config (main.app.get_config_file ())
		connectwin.ConnectWin (self.create_bookmark, config_opts)

	def create_bookmark (self, connectwin):

		opts = connectwin.get_options ()
		if opts['host']:
			bookfile = os.path.join (self.get_path_to_current_node (), opts['host'])
			self.save_bookmark (bookfile, opts)
			self.display_bookmarks ()

	def edit_bookmark (self):

		path = self.get_path_to_current_node ()
		name = self.list.get_text (self.list.selection.pop (0), 0)
		bookfile = os.path.join (path, name)
		options = self.load_bookmark (bookfile)

		def after_edit (connectwin, bookfile=bookfile, self=self):

			opts = connectwin.get_options ()
			self.save_bookmark (bookfile, opts)

		connectwin.ConnectWin (after_edit, options)

	def rename_bookmark (self):

		path = self.get_path_to_current_node ()
		name = self.list.get_text (self.list.selection.pop (0), 0)
		new_name = GtkExtra.input_box (title="FtpCube - Rename Bookmark", message="Rename to:")
		if new_name is not None:
			try:
				os.rename (os.path.join (path, name), os.path.join (path, new_name))
			except OSError, (errno, strerror):
				GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ]))
			self.display_bookmarks ()

	def remove_bookmark (self):

		if not self.list.selection:
			# Nothing selected so do nothing
			return

		path = self.get_path_to_current_node ()
		name = self.list.get_text (self.list.selection.pop (0), 0)
		path = os.path.join (path, name)
		try:
			os.unlink (path)
		except OSError, (errno, strerror):
			GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ]))
			return
		self.display_bookmarks ()

	def save_bookmark (self, bookmark, opts):

		file = open (bookmark, 'w')
		pickler = pickle.Pickler (file)
		pickler.dump (opts)
		file.close ()

		# Set the permission of the bookmark to be sure it's private
		try:
			os.chmod (bookmark, 0600)
		except OSError, strerror:
			GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ] ))

	def load_bookmark (self, file):

		file = open (file)
		pickler = pickle.Unpickler (file)
		bookmark = pickler.load ()
		file.close ()
		return bookmark

	def load_bookmark_tree (self, path, parent=None):

		try:
			files = os.listdir (path)
		except OSError, (errno, strerror):
			GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ]))
			return

		if not files:
			# Then our parent is really a leaf folder
			self.tree.toggle_expansion (parent)
		else:
			# Add our children into the tree
			for file in files:
				full_path = os.path.join (path, file)
				if os.path.isdir (full_path):
					node = self.tree.insert_node (parent, None, [ file ],
						pixmap_closed = self.folder_pic,
						mask_closed = self.folder_mask,
						pixmap_opened = self.folder_pic,
						mask_opened = self.folder_mask,
						is_leaf = gtk.FALSE)
					self.load_bookmark_tree (full_path, parent=node)

	def display_bookmarks (self):

		path = self.get_path_to_current_node ()

		try:
			files = os.listdir (path)
		except OSError, (errno, strerror):
			GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ]))
			return

		self.list.freeze ()
		self.list.clear ()

		for file in files:
			full_path = os.path.join (path, file)
			if os.path.isfile (full_path):
				bookmark = self.load_bookmark (full_path)
				self.list.append ((file, "%s:%s" %(bookmark['host'], bookmark['port']),
				                   bookmark['username'], bookmark['remotedir']))

		self.list.thaw ()
