#
# FtpCube
# Copyright (C) 2001 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import gtk
import main

class CmdWin:

	def __init__ (self, container):

		self.format = {
			'local'  : "LOCAL >  %s\n",
			'remote' : "REMOTE> %s\n",
			'error'  : "! - %s\n"
		}

		self.colors = {
			'local' : None,
			'remote' : None,
			'error' : None
		}

		self.gdk_colors = {
			'local' : None,
			'remote' : None,
			'error' : None
		}

		table = gtk.GtkTable (2, 1)
		container.add (table)
		table.show ()

		self.text = gtk.GtkText ()
		self.text.set_editable (gtk.FALSE)
		table.attach (self.text, 0, 1, 0, 1)
		self.text.show ()

		vadj = self.text.get_vadjustment ()
		self.scroll = gtk.GtkVScrollbar (vadj)
		table.attach (self.scroll, 1, 2, 0, 1, xoptions=gtk.FILL)
		self.scroll.show ()
		vadj.connect ("changed", self.scroll_check)

		self.colormap = self.text.get_colormap()

	def scroll_check (self, adj):

		if adj.upper - adj.lower <= adj.page_size:
			self.scroll.hide()
		else:
			self.scroll.show()

	def display_banner (self):

		self.insert_color_text ('local', 'Welcome to FtpCube v0.2.0')
		self.insert_color_text ('local', 'Copyright (C) 2001  Michael Gilfix')
		self.insert_color_text ('local', '==========================================')
		self.insert_color_text ('local', '')

	def insert_color_text(self, kind, text):

		gdk_color = self.get_gdk_color(kind)
		text = self.format[kind] % text
		self.text.insert (None, gdk_color, None, text)

	def get_gdk_color(self, kind):

		current_color = main.app[kind + '_color']

		if self.colors[kind] == current_color:
			gdk_color = self.gdk_colors[kind]
		else:
			rgbo = map (lambda c: int (c * 65535.0), current_color)
			gdk_color = apply (self.colormap.alloc, rgbo)
			self.colors[kind] = current_color
			self.gdk_colors[kind] = gdk_color

		return gdk_color
