#
# FtpCube
# Copyright (C) 2001 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import gtk, GTK, GtkExtra
import string # For backwards compatibility with 1.5.x
import os.path
import main, app, ftp, threads

class CtrlWin:

	def __init__ (self, container):

		box = gtk.GtkVBox ()
		box.set_usize (275, -1)
		box.show ()
		container.add (box)

		self.notebook = gtk.GtkNotebook ()
		self.notebook.set_tab_pos (gtk.POS_TOP)
		box.pack_start (self.notebook)
		self.notebook.show ()

		status_box = gtk.GtkHBox ()
		status_box.set_usize (-1, 15)
		status_box.show ()
		box.pack_start (status_box, expand=gtk.FALSE)

		self.queue_tab = QueueTab (self)
		label = gtk.GtkLabel ("Queue")
		label.set_padding (2, 2)
		label.show ()
		self.notebook.append_page (self.queue_tab.get_page (), label)

		self.threads_tab = ThreadsTab (self)
		label = gtk.GtkLabel ("Threads")
		label.set_padding (2, 2)
		label.show ()
		self.notebook.append_page (self.threads_tab.get_page (), label)

		self.download_tab = DownloadTab (self)
		label = gtk.GtkLabel ("Downloads")
		label.set_padding (2, 2)
		label.show ()
		self.notebook.append_page (self.download_tab.get_page (), label)

		self.failure_tab = FailureTab (self)
		label = gtk.GtkLabel ("Failures")
		label.set_padding (2, 2)
		label.show ()
		self.notebook.append_page (self.failure_tab.get_page (), label)

		# Default to the thread page
		self.notebook.set_page (1)

	def add_to_queue (self, hash):

		for name, size, date, flags, truesize in hash['files']:
			self.queue_tab.add_to_queue ({
				'file'        : name,
				'server'      : hash['server'],
				'port'        : hash['port'],
				'username'    : hash['username'],
				'password'    : hash['password'],
				'remote_path' : hash['remote_path'],
				'local_path'  : hash['local_path'],
				'size'        : truesize,
				'attempt'     : 1,
				'maxattempts' : hash['maxattempts'],
				'direction'   : hash['direction'],
				'flags'       : flags,
			})

	def add_to_downloads (self, file):
		self.download_tab.add_to_list (file)

	def attach_main_thread (self, thread):
		self.threads_tab.attach_main_thread (thread)

	def get_thread_list (self):
		return self.threads_tab.get_thread_list ()

class QueueTab:

	headings = [ 'Host', 'Filename', 'Size', 'Attempt', 'Direction', 'Port',
	             'Username', 'Password', 'Remote Path', 'Local Path', 'Type' ]

	def __init__ (self, ctrl):

		# Initialize instance variables
		self.ctrlwin = ctrl
		self.disabled = 0                # For enabling and disabling the queueing process

		self.page = gtk.GtkVBox ()
		self.page.show ()

		box = gtk.GtkHBox ()
		box.show ()
		toolbar = gtk.GtkToolbar (gtk.ORIENTATION_HORIZONTAL, gtk.TOOLBAR_ICONS)
		toolbar.set_space_size (10)
		toolbar.set_space_style (GTK.TOOLBAR_SPACE_LINE)
		toolbar.show ()
		box.pack_start (toolbar, padding=6)
		self.page.pack_start (box, expand=gtk.FALSE, padding=4)

		enable_pic, enable_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "thread_idle.xpm"))
		clear_pic, clear_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "clear.xpm"))
		load_pic, load_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "load_queue.xpm"))
		save_pic, save_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "save_queue.xpm"))

		toolbar.append_element (GTK.TOOLBAR_CHILD_TOGGLEBUTTON, None,
			"", "Enable Queuing Process", "Enable Queuing Process",
			gtk.GtkPixmap (enable_pic, enable_mask), self.enable_queuing_process)
		toolbar.append_space ()
		toolbar.append_item ("", "Clear Queue", "Clear Queue",
			gtk.GtkPixmap (clear_pic, clear_mask),
			self.clear_queue)
		toolbar.append_space ()
		toolbar.append_item ("", "Load Queue", "Load Queue",
			gtk.GtkPixmap (load_pic, load_mask),
			self.load_queue)
		toolbar.append_item ("", "Save Queue", "Save Queue",
			gtk.GtkPixmap (save_pic, save_mask),
			self.save_queue)

		win = gtk.GtkScrolledWindow ()
		win.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		win.show ()
		self.page.pack_start (win)

		self.list = gtk.GtkCList (len (self.headings), self.headings)
		self.list.connect ("select_row", self.select_items)
		self.list.set_column_width (0, 80)
		self.list.set_column_width (1, 150)
		self.list.set_column_width (2, 80)
		self.list.set_column_width (3, 60)
		# Make the invisible ones truly invisible
		for i in range (4,11):
			self.list.set_column_visibility (i, 0)
		self.list.set_selection_mode (GTK.SELECTION_MULTIPLE)
		self.list.show ()
		win.add_with_viewport (self.list)

	def enable_queuing_process (self, button):

		if not self.disabled:
			self.disabled = 1
		else:
			self.disabled = 0

	def add_to_queue (self, hash):

		self.list.append ((hash['server'], hash['file'], "%s" %hash['size'], "%s/%s" %(hash['attempt'], hash['maxattempts']),
		                   "%s" %hash['direction'], "%s" %hash['port'], hash['username'], hash['password'], hash['remote_path'],
		                   hash['local_path'], hash['flags']))
		if not self.disabled:
			self.ctrlwin.threads_tab.check_if_available_threads ()

	def get_next (self):

		if not self.length ():
			return None

		# Build the first row entry
		entry = [ ]
		for i in range (self.list.columns):
			entry.append (self.list.get_text (0, i))
		return tuple (entry)

	def remove_next (self):

		self.list.freeze ()
		self.list.remove (0)
		self.list.thaw ()

	def length (self):

		# Check if we're disabled and return a zero length to calling
		# functions will think we're empty
		if self.disabled:
			return 0
		else:
			return self.list.rows

	def clear_queue (self, button):
		self.list.clear ()

	def save_queue (self, button):

		path = GtkExtra.file_sel_box (title="FtpCube - Save queue file")
		if path is None:
			return

		try:
			file = open (path, 'w')
		except IOError, strerror:
			GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ] ))
			return

		for r in range (self.list.rows):
			entry = [ ]
			for c in range (self.list.columns):
				entry.append (self.list.get_text (r, c))
			entry = string.join (entry, '|')
			file.write ("%s\n" %entry)
		file.close ()

	def load_queue (self, button):

		path = GtkExtra.file_sel_box ("FtpCube - Load queue file")
		if path is None:
			return

		try:
			file = open (path, "r")
		except IOError, strerror:
			GtkExtra.message_box (title="FtpCube Error", message="FtpCube Error: %s" %strerror, buttons=( [ "Ok" ] ))
			return

		line = file.readline ()
		while line:
			entry = string.split (line, '|')
			self.list.append (list (entry))
			line = file.readline ()
		file.close ()

	def select_items (self, list, r, c, event):
		pass

	def get_page (self):
		return self.page

class ThreadsTab:

	def __init__ (self, ctrl):

		# Initialize instance variables
		self.threads = [ ]
		self.main_thread = None
		self.main_transfer = 0
		self.ctrlwin = ctrl

		self.page = gtk.GtkVBox ()
		self.page.show ()

		box = gtk.GtkHBox ()
		box.show ()
		toolbar = gtk.GtkToolbar (gtk.ORIENTATION_HORIZONTAL, gtk.TOOLBAR_ICONS)
		toolbar.set_space_size (10)
		toolbar.set_space_style (GTK.TOOLBAR_SPACE_LINE)
		toolbar.show ()
		box.pack_start (toolbar, padding=6)
		self.page.pack_start (box, expand=gtk.FALSE, padding=4)

		box = gtk.GtkHBox ()
		toolbar.append_widget (box, "", "")
		box.show ()
		self.no_threads = gtk.GtkSpinButton (gtk.GtkAdjustment (0, 0, 16, 1, 10, 0), 1, 0)
		self.no_threads.set_numeric (gtk.TRUE)
		self.no_threads.set_value (3.0)
		self.no_threads.show ()
		box.pack_start (self.no_threads)
		label = gtk.GtkLabel ("max threads")
		label.show ()
		box.pack_start (label)

		toolbar.append_space ()
		pic, mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "main_thread.xpm"))
		toolbar.append_element (GTK.TOOLBAR_CHILD_TOGGLEBUTTON, None,
			"", "Only use main thread for transfers",
			"Only use main thread for transfers",
			gtk.GtkPixmap (pic, mask),
			self.use_main_thread)

		self.transfer_label = gtk.GtkLabel ()
		self.update_transfer_totals (0, 0.0)
		self.transfer_label.set_line_wrap (gtk.TRUE)
		self.transfer_label.show ()
		self.page.pack_start (self.transfer_label, expand=gtk.FALSE)

		frame = gtk.GtkFrame ()
		frame.set_border_width (10)
		frame.set_shadow_type (gtk.SHADOW_ETCHED_OUT)
		frame.show ()
		self.page.pack_start (frame)
		self.thread_box = gtk.GtkPacker ()
		self.thread_box.show ()
		frame.add (self.thread_box)

		# Add our idle counting function
		gtk.timeout_add (1000, self.idle_count)

	def update_transfer_totals (self, threads, speed):
		self.transfer_label.set_text ("%d Threads running, total transfer speed is %.02f Kb/s" %(threads, speed))

	def attach_main_thread (self, thread):

		self.main_thread = thread
		self.thread_box.add (thread.get_entry (), anchor=GTK.ANCHOR_N, options=GTK.FILL_X+GTK.FILL_Y)

	def use_main_thread (self, button):

		# Set/unset the flag in one fell swoop
		self.main_transfer = (self.main_transfer + 1) %2

	def check_if_available_threads (self):

		thread_entry = None
		server, file, size, attempt, direction, port, username, password, remote_path, local_path, flags = self.ctrlwin.queue_tab.get_next ()

		if self.main_transfer:
			if not self.main_thread.busy ():
				# Check if this file is on the same host. Otherwise, spawn a
				# transfer thread, since we don't really care about having multiple
				# connections to different hosts
				if self.main_thread.get_host () == server and \
				   self.main_thread.get_port () == int (port):
					thread_entry = self.main_thread
				else:
					thread_entry = self.create_thread (server, int (port), username, password)
		else:
			# First check if there's an idling thread
			found = 0

			for t in self.threads:
				if not t.busy ():
					found = 1
					# Check if we're still connected to the same host. If not, cancel
					# the thread and create a new one connected to the right host
					if t.get_host () == server and t.get_port () == int (port):
						thread_entry = t
					else:
						t.cancel_thread ()
						thread_entry = self.create_thread (server, int (port), username, password)
					break

			if not found:
				if len (self.threads) < self.no_threads.get_value ():
					thread_entry = self.create_thread (server, int (port), username, password)

		if thread_entry is not None:
			self.ctrlwin.queue_tab.remove_next ()

			if flags[0] == 'f':
				thread_entry.initiate_transfer (remote_path, local_path, file, int (size), int (direction))
			elif flags[0] == 'd':
				thread_entry.recurse_directory (remote_path, local_path, file, int (size), int (direction))
			else:
				thread_entry.recurse_link (remote_path, local_path, file, int (size), int (direction))

	def create_thread (self, server, port, username, password):

		# Use our app as default opts
		opts = { }
		for key in main.app.keys ():
			opts[key] = main.app[key]

		opts['host'] = server
		opts['port'] = port
		opts['username'] = username
		opts['password'] = password

		thread_entry = threads.TransferThread (opts)
		self.add_thread_entry (thread_entry)
		return thread_entry

	def get_truesize (self, file, direction):

		if direction == ftp.DOWNLOAD:
			return main.app.remotewin.get_file_size (file)
		elif direction == ftp.UPLOAD:
			return main.app.localwin.get_file_size (file)
		return None

	def add_thread_entry (self, entry):

		self.threads.append (entry)
		self.thread_box.add (entry.get_entry (), anchor=GTK.ANCHOR_N, options=GTK.FILL_X+GTK.FILL_Y)

	def idle_count (self):

		total_rate = 0.0
		for t in self.threads:
			if not t.busy ():
				t.update_idle ()
				if t.finished ():
					self.thread_box.remove (t.get_entry ())
					self.threads.remove (t)
			elif t.isTransfering ():
				t.update_transfer_estimate ()
				total_rate = total_rate + t.transfer_speed ()

		# First check if there's remaining in the queue
		# If so, feed it into the threads
		if self.ctrlwin.queue_tab.length ():
			self.check_if_available_threads ()

		no_threads = len (self.threads)

		# Now check the main thread
		if self.main_thread is not None:
			# Count the main thread
			no_threads = no_threads + 1

			if self.main_thread.finished ():
				self.thread_box.remove (self.main_thread.get_entry ())
				# Set the thread references to None
				self.main_thread = None
				main.app.main_thread = None
			elif not self.main_thread.busy ():
				self.main_thread.update_idle ()

		# Now update the totals
		self.update_transfer_totals (no_threads, total_rate)
		return gtk.TRUE

	def get_thread_list (self):

		if self.main_thread is not None:
			thread_list = [ self.main_thread ]
			thread_list.extend (self.threads)
		else:
			thread_list = self.threads
		return thread_list

	def get_page (self):
		return self.page

class DownloadTab:

	status_headings = [ 'Filename', 'Status', 'Type', 'Size', 'Local Path' ]
	archive_headings = [ 'Filename', 'Size' ]

	def __init__ (self, ctrl):

		self.ctrlwin = ctrl

		self.page = gtk.GtkVBox ()
		self.page.show ()

		box = gtk.GtkHBox ()
		box.show ()
		toolbar = gtk.GtkToolbar (gtk.ORIENTATION_HORIZONTAL, gtk.TOOLBAR_ICONS)
		toolbar.set_space_size (10)
		toolbar.show ()
		box.pack_start (toolbar, padding=6)
		self.page.pack_start (box, expand=gtk.FALSE, padding=4)

		clear_pic, clear_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "clear.xpm"))
		archive_pic, archive_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "check_archive.xpm"))

		toolbar.append_item ("", "Clear List", "Clear List",
			gtk.GtkPixmap (clear_pic, clear_mask),
			self.clear_list)
		toolbar.append_item ("", "Show Archive Details", "Show Archive Details",
			gtk.GtkPixmap (archive_pic, archive_mask),
			self.show_archive)
		toolbar.append_space ()

		box = gtk.GtkHBox ()
		box.show ()
		self.max_attempts = gtk.GtkSpinButton (gtk.GtkAdjustment (0, 0, 16, 1, 10, 0), 1, 0)
		self.max_attempts.set_numeric (gtk.TRUE)
		self.max_attempts.set_value (3.0)
		self.max_attempts.show ()
		box.pack_start (self.max_attempts)
		label = gtk.GtkLabel ("max attempts")
		label.show ()
		box.pack_start (label, padding=4)
		toolbar.append_widget (box, "", "")

		win = gtk.GtkScrolledWindow ()
		win.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		win.show ()
		self.page.pack_start (win)

		self.status_list = gtk.GtkCList (len (self.status_headings), self.status_headings)
		self.status_list.connect ("select_row", self.select_status_items)
		for i in range (len (self.status_headings)):
			self.status_list.set_column_width (i, 75)
		# Explicitly make the filename column bigger
		self.status_list.set_column_width (0, 125)
		self.status_list.set_column_visibility (4, 0)
		self.status_list.set_selection_mode (GTK.SELECTION_MULTIPLE)
		self.status_list.show ()
		win.add_with_viewport (self.status_list)

		win = gtk.GtkScrolledWindow ()
		win.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		win.set_usize (-1, 200)
		win.show ()
		self.page.pack_start (win, expand=gtk.FALSE)

		self.archive_list = gtk.GtkCList (len (self.archive_headings), self.archive_headings)
		self.archive_list.connect ("select_row", self.select_archive_items)
		self.archive_list.set_column_width (0, 150)
		self.archive_list.set_selection_mode (GTK.SELECTION_MULTIPLE)
		self.archive_list.show ()
		win.add_with_viewport (self.archive_list)

	def clear_list (self, button):
		self.status_list.clear ()

	def show_archive (self, button):
		pass

	def select_status_items (self, list, r, c, event):
		pass

	def select_archive_items (self, list, r, c, event):
		pass

	def add_to_list (self, file):

		(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size,
		 st_atime, st_mtime, st_ctime) = os.stat (file)
		size = app.beautify_size (st_size)

		# Note that file is a full pathname so let's split it for the list
		path, filename = os.path.split (file)

		# For now, just put status and type to "OK" and "Unknown"
		# Will need to fix later
		self.status_list.append ( (filename, "OK", "Unknown", size, path) )

	def get_page (self):
		return self.page

class FailureTab:

	headings = [ 'Server', 'Filename', 'Size', 'Error Msg', 'Direction', 'Port', 'Username', 'Password', 'Remote Path', 'Local Path', 'Type' ]

	def __init__ (self, ctrl):

		self.ctrlwin = ctrl

		self.page = gtk.GtkVBox ()
		self.page.show ()

		box = gtk.GtkHBox ()
		box.show ()
		toolbar = gtk.GtkToolbar (gtk.ORIENTATION_HORIZONTAL, gtk.TOOLBAR_ICONS)
		toolbar.set_space_size (10)
		toolbar.show ()
		box.pack_start (toolbar, padding=6)
		self.page.pack_start (box, expand=gtk.FALSE, padding=4)

		clear_pic, clear_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "clear.xpm"))
		resubmit_pic, resubmit_mask = gtk.create_pixmap_from_xpm (toolbar, None, os.path.join (main.ICONS_PREFIX, "resubmit_job.xpm"))

		toolbar.append_item ("", "Clear List", "Clear List",
			gtk.GtkPixmap (clear_pic, clear_mask),
			self.clear_list)
		toolbar.append_item ("", "Resubmit Job", "Resubmit Job",
			gtk.GtkPixmap (resubmit_pic, resubmit_mask),
			self.resubmit_job)

		win = gtk.GtkScrolledWindow ()
		win.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		win.show ()
		self.page.pack_start (win)

		self.list = gtk.GtkCList (len (self.headings), self.headings)
		for i in range (len (self.headings)):
			self.list.set_column_width (i, 80)
		# Explicitly make the filename bigger
		self.list.set_column_width (1, 100)
		# Make the invisible, well, invisible
		for i in range (4, 11):
			self.list.set_column_visibility (i, 0)
		self.list.set_selection_mode (GTK.SELECTION_MULTIPLE)
		self.list.show ()
		win.add_with_viewport (self.list)

	def clear_list (self, button):
		self.list.clear ()

	def resubmit_job (self, button):

		for i in self.list.selection:

			server, file, size, error, direction, port, username, password, remote_path, local_path, type = self.get_list_entry (i)
			self.ctrlwin.queue_tab.add_to_queue ({
				'file'        : file,
				'server'      : server,
				'port'        : port,
				'username'    : username,
				'password'    : password,
				'remote_path' : remote_path,
				'local_path'  : local_path,
				'size'        : size,
				'attempt'     : 1,
				'maxattempts' : self.ctrlwin.threads_tab.main_thread.get_retries (),
				'direction'   : direction,
				'flags'       : type
			})

	def get_list_entry (self, pos):

		entry = [ ]
		for i in range (self.list.columns):
			entry.append (self.list.get_text (pos, i))
		return tuple (entry)

	def add_to_list (self, hash):

		self.list.append ((hash['server'], hash['file'], hash['size'], hash['error'], hash['direction'],
		                   hash['port'], hash['username'], hash['password'], hash['remote_path'],
		                   hash['local_path'], hash['type']))

	def get_page (self):
		return self.page
